# Imports from python.
import json


# Imports from Django.
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import HttpResponse
from django.urls import reverse_lazy
from django.views.generic import DetailView
from django.views.generic.edit import CreateView
from django.views.generic.edit import UpdateView
from django.views.generic.list import ListView


# Imports from other dependencies.
from editorial_staff.models import Staffer


# Imports from assignment-desk.
from assignment_desk.forms import InlineAssignmentFormset
from assignment_desk.forms import WeekCreationForm
from assignment_desk.forms import WeekEditingForm
from assignment_desk.mixins import InlineFormsetMixin
from assignment_desk.mixins import NavigationContextMixin
from assignment_desk.models import Assignment
from assignment_desk.models import Role
from assignment_desk.models import Week


DAYS_IN_WEEK = [
    'monday',
    'tuesday',
    'wednesday',
    'thursday',
    'friday',
    'saturday',
    'sunday',
]


def index_view(request):
    return HttpResponse('Hello, world. You\'re at the assignment-desk index.')


class WeekListView(LoginRequiredMixin, NavigationContextMixin, ListView):
    queryset = Week.objects.all()
    template_name = 'assignment_desk/weeks/list.html'


class WeekCreateView(LoginRequiredMixin, NavigationContextMixin, CreateView):
    model = Week
    form_class = WeekCreationForm
    template_name = 'assignment_desk/weeks/create_form.html'


class WeekEditView(LoginRequiredMixin, NavigationContextMixin,
                   InlineFormsetMixin, UpdateView):
    model = Week
    template_name = 'assignment_desk/weeks/edit_form.html'

    form_class = WeekEditingForm
    inline_formset_class = InlineAssignmentFormset

    def get_queryset(self):
        return Week.objects.prefetch_related(
            'assignments',
            # 'assignments__staffer',
            'assignments__role',
            'assignments__role__type',
        )

    def get_context_data(self, **kwargs):
        initial_context = super(WeekEditView, self).get_context_data(**kwargs)

        initial_context['role_crosswalk'] = {
            assignment.role.id: {
                'name': assignment.role.name,
                'slug': assignment.role.slug,
            }
            for assignment in self.object.assignments.all()
        }

        raw_staffers = [
            {
                'label': staffer.full_name,
                'value': staffer.email,
                'customProperties': {
                    'id': staffer.id,
                    'firstName': staffer.first_name,
                    'lastName': staffer.last_name,
                    'fullName': staffer.full_name,
                    'formattedName': staffer.render_formatted_name(),
                    'email': staffer.email,
                    'imageURL': staffer.image_url,
                    'active': True,
                },
            } for staffer in Staffer.objects.filter(active=True)
        ]

        initial_context['staffers'] = json.dumps(raw_staffers)

        initial_context['staff_by_id'] = {
            staffer_object['customProperties']['id']: {
                'fullName': staffer_object['customProperties']['fullName'],
                'firstName': staffer_object['customProperties']['firstName'],
                'lastName': staffer_object['customProperties']['lastName'],
                'imageURL': staffer_object['customProperties']['imageURL'],
            } for staffer_object in raw_staffers
        }

        return initial_context

    def get_success_url(self):
        submit_action = self.request.POST.get('submit', '_save')

        if submit_action == '_continue':
            return reverse_lazy(
                'assignment-desk:week-edit',
                kwargs={'pk': self.object.pk}
            )

        return reverse_lazy('assignment-desk:week-list')


class WeekDetailView(LoginRequiredMixin, NavigationContextMixin, DetailView):
    queryset = Week.objects.all().prefetch_related(
        'assignments',
        'assignments__role',
        'assignments__staffer',
    )
    template_name = 'assignment_desk/weeks/detail.html'

    def get_context_data(self, **kwargs):
        context = super(WeekDetailView, self).get_context_data(**kwargs)

        context['role_crosswalk'] = {
            assignment.role.id: {
                'name': assignment.role.name,
                'slug': assignment.role.slug,
            }
            for assignment in self.object.assignments.all()
        }

        return context



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-08-09 19:32
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('assignment_desk', '0003_auto_20180615_1551'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='role',
            options={'ordering': ['priority', 'name']},
        ),
        migrations.AlterModelOptions(
            name='roletype',
            options={'ordering': ['priority', 'name']},
        ),
        migrations.AlterField(
            model_name='assignment',
            name='staffer',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='assignments', to='editorial_staff.Staffer'),
        ),
    ]



# Imports from python.
from datetime import timedelta


# Imports from Django.
from django.db.models.signals import post_save
from django.db import transaction
from django.dispatch import receiver


# Imports from assignment-desk.
from assignment_desk.models import Assignment
from assignment_desk.models import Week


@receiver(post_save, sender=Week, dispatch_uid="populate_assignments")
def create_assignments_for_empty_week(sender, instance, **kwargs):
    if instance.assignments.count() == 0 and instance.role_type:
        roles_to_add = []

        for role in instance.role_type.roles.all():
            roles_to_add.extend([
                Assignment(
                    week_id=instance.id,
                    role_id=role.id,
                    day=(instance.beginning_date + timedelta(days=day_delta))
                ) for day_delta in range(7)
            ])

        with transaction.atomic():
            Assignment.objects.bulk_create(roles_to_add)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.apps import AppConfig


class AssignmentDeskConfig(AppConfig):
    name = 'assignment_desk'

    def ready(self):
        import assignment_desk.signals



# Imports from python.  # NOQA
from datetime import datetime
from datetime import timedelta


def get_latest_monday():
    current_date = datetime.now().date()
    return current_date - timedelta(days=current_date.weekday())


def get_next_sunday():
    current_date = datetime.now().date()
    start_of_week = current_date - timedelta(days=current_date.weekday())
    return start_of_week + timedelta(days=6)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-08-14 19:38
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('assignment_desk', '0004_auto_20180809_1432'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='assignment',
            options={'ordering': ['role', 'day']},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-06-14 18:21
from __future__ import unicode_literals

import assignment_desk.utils
import colorfield.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('editorial_staff', '0007_populate_staffer_full_names'),
    ]

    operations = [
        migrations.CreateModel(
            name='Assignment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('day', models.DateField()),
                ('notes', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('slug', models.SlugField()),
                ('description', models.TextField(blank=True, null=True)),
                ('color', colorfield.fields.ColorField(default='#0185D3', max_length=18)),
                ('priority', models.PositiveSmallIntegerField(default=10)),
            ],
        ),
        migrations.CreateModel(
            name='RoleType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('slug', models.SlugField()),
                ('priority', models.PositiveSmallIntegerField(default=10)),
            ],
        ),
        migrations.CreateModel(
            name='Week',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('beginning_date', models.DateField(default=assignment_desk.utils.get_latest_monday)),
                ('ending_date', models.DateField(default=assignment_desk.utils.get_next_sunday)),
                ('extra_information', models.TextField(blank=True, null=True)),
                ('assigned_staffers', models.ManyToManyField(through='assignment_desk.Assignment', to='editorial_staff.Staffer')),
            ],
        ),
        migrations.AddField(
            model_name='role',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='assignment_desk.RoleType'),
        ),
        migrations.AddField(
            model_name='assignment',
            name='role',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='assignment_desk.Role'),
        ),
        migrations.AddField(
            model_name='assignment',
            name='staffer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='editorial_staff.Staffer'),
        ),
        migrations.AddField(
            model_name='assignment',
            name='week',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='assignment_desk.Week'),
        ),
    ]
